.include "m48def.inc"
.def temp=r16
.def temp1=r17
.def counter=r20
.def cntbit=r22
.equ reslcd=pc5	;pin 28(dip) 28(tqfp)
.equ sce   =pc4 ;pin 27      27
.equ dc    =pc3 ;    26      26
.equ sdin  =pc2 ;    25      25
.equ sclk  =pc1 ;    24      24
.equ scr_ram_start=$100	; screen RAM - $0100-$0153 (14x6=84 bytes)
;
;   -----------------------------
;  | 1 | 2| 3| ............ | 14 |
;  |-----------------------------| <- LCD (symbol numbers)
;  | 15|16|17| .............| 28 |
;  |  .                          |
;  |  . ....................     |
;  |-----------------------------|
;  | 71|                    | 84 |
;   -----------------------------
;
;        --->
;       000000      coding method for every symbol(6x8)
;     ^ 011100
;     | 100010
;     | 100010
;     | 100010    example: $fc-$22-$22-$22-$fc-$00 (A)
;     | 111110
;     | 100010
;       100010

.org $0000
		rjmp start
.org $50
start:	
		ldi temp,high(ramend) 		;Stack init
		out sph,temp
		ldi temp,low(ramend) 
		out spl,temp
;-------PORTs init-------------------------------
		ldi temp,(1<<reslcd)+(1<<sce)+(1<<dc)+(1<<sdin)+(1<<sclk)
		out ddrc,temp
		out portc,temp
;-------LCD init---------------------------------
		cbi portc,sclk
		cbi portc,sce
		nop
		cbi portc,reslcd			;RESET LCD don't forget!!!
		nop
		nop
		nop
		nop
		sbi portc,reslcd
		cbi portc,dc				;dc=0 ->write command to LCD
		ldi temp,0b00100001;$21		;func set H=1,active mode
		rcall outbyte
		ldi temp,0b11000010			;set Vop=1001010 (for 5V!!!)
		rcall outbyte
		ldi temp,0b00100000;$20		;H=0
		rcall outbyte
		ldi temp,0b00001100;$0d		;normal mode - 00001100
									;inverse mode- 00001101
		rcall outbyte
		ldi temp,0b10000000;$80		;start Xadr=0 (1st line, 1st position)
		rcall outbyte
;-----init is over. Start data writing----------------------------
		sbi portc,dc				;dc=1 ->write data to LCD RAM
		ldi zl,low(example*2)		;load start adress in ROM(here just an example...)
		ldi zh,high(example*2)
		ldi xl,low(scr_ram_start)	;x - screenRAM index 
		ldi xh,high(scr_ram_start)
;---fill 84 bytes with symbol codes-------------------------------
		ldi counter,14*6			;14*6=84 symbols in LCD
bb:		lpm							;load symbol with Z-index (to R0)
		st x+,r0					;store it in screenRAM
		adiw z,1					;inc index
		dec counter					;all symbols?
		brne bb						;NO, get next
;---return start adr and counter----------------------------------
		ldi counter,14*6
		ldi xl,low(scr_ram_start)
		ldi xh,high(scr_ram_start)
;-----------------------------------------------------------------				
kk:		rcall outsym				;output 1 symbol 6x8 pixels
		dec counter					;all symbols?
		brne kk
kk1:	rjmp kk1					;yes! stop.
;-----------------------------------------------------------------
outsym:	ldi zl,low(font*2)			;now our target is FONT!
		ldi zh,high(font*2)
		ld temp,x+					;load symbol code
		subi temp,$20				;offset=((sym code)-$20)*6
		ldi temp1,6					;temp1*temp->r1:r0
		mul temp,temp1
		add zl,r0					;z=z+offset(r1:r0)
		adc zh,r1					;find address in font table(in Z)
		ldi cntbit,6				;cntbit - byte counter(6 bytes in 1 symbol)
l1:		lpm							;get 1 byte from table (addr Z)
		mov temp,r0					;put it to R0 and R16
		rcall outbyte				;out - 1 byte
		adiw z,1					;inc address
		dec cntbit					
		brne l1 					;all bytes? NO - take next
		ret
;-----------------------------------------------------------------
outbyte:ldi temp1,8					;quantity of bits in byte
nextbit:rol temp					;left shift to carry
		brcc zero
		sbi portc,sdin				;if C=1, set SDIN
		rjmp go
zero:	cbi portc,sdin				;if C=0, clear SDIN
go:		nop							;short delay
		sbi portc,sclk				;strobe
		nop
		cbi portc,sclk
		dec temp1					;all bits are shifted?
		brne nextbit				;NO - take the next one.
		ret

example:
.db " 1KHz Serial   R= 178.643",$7c,"   C= 127.789pF  L=----.---mH  Q=100 DF=  3 K12 Ubat=11.4V"

;-----------------------------------------------------------------
;   symbol codes (standart)  offset = ((symb.code)-20)*6
font:
.db $00,$00,$00,$00,$00,$00;  (20H)  symbol place number 
.db	$00,$00,$5f,$00,$00,$00;!
.db	$00,$07,$00,$07,$00,$00;"
.db	$14,$7f,$14,$7f,$14,$00;#
.db $04,$2a,$7f,$2a,$10,$00;$
.db $23,$13,$08,$64,$62,$00;%
.db $34,$4a,$54,$20,$50,$00;&
.db $00,$05,$03,$00,$00,$00;'
.db $00,$1c,$22,$41,$00,$00;(
.db $00,$41,$22,$1c,$00,$00;)
.db $14,$08,$3e,$08,$14,$00;*
.db $08,$08,$3e,$08,$08,$00;+
.db $00,$00,$50,$30,$00,$00;,
.db $10,$10,$10,$10,$10,$00;-
.db $00,$60,$60,$00,$00,$00;.
.db $20,$10,$08,$04,$02,$00;/
.db $3e,$51,$49,$45,$3e,$00;0 (30H)
.db $00,$42,$7f,$40,$00,$00;1
.db $42,$61,$51,$49,$46,$00;2
.db $21,$41,$45,$4b,$31,$00;3
.db $18,$14,$12,$7f,$10,$00;4
.db $27,$45,$45,$45,$39,$00;5
.db $3c,$4a,$49,$49,$30,$00;6
.db $01,$71,$09,$05,$03,$00;7
.db $36,$49,$49,$49,$36,$00;8
.db $06,$49,$49,$29,$1e,$00;9
.db $00,$36,$36,$00,$00,$00;:
.db $00,$56,$36,$00,$00,$00;;
.db $08,$14,$22,$41,$00,$00;<
.db $14,$14,$14,$14,$14,$00;=
.db $00,$41,$22,$14,$08,$00;>
.db $02,$01,$51,$09,$06,$00;?
.db $32,$49,$59,$51,$3e,$00;@
.db $7e,$11,$11,$11,$7e,$00;A
.db $7f,$49,$49,$49,$36,$00;B
.db $3e,$41,$41,$41,$22,$00;C
.db $7f,$41,$41,$22,$1c,$00;D
.db $7f,$49,$49,$49,$41,$00;E
.db $7f,$09,$09,$09,$01,$00;F
.db $3e,$41,$49,$49,$7a,$00;G
.db $7f,$08,$08,$08,$7f,$00;H
.db $00,$41,$7f,$41,$00,$00;I
.db $20,$40,$41,$3f,$01,$00;J
.db $7f,$08,$14,$22,$41,$00;K
.db $7f,$40,$40,$40,$40,$00;L
.db $7f,$02,$0c,$02,$7f,$00;M
.db $7f,$04,$08,$10,$7f,$00;N
.db $3e,$41,$41,$41,$3e,$00;O
.db $7f,$09,$09,$09,$06,$00;P
.db $3e,$41,$51,$21,$5e,$00;Q
.db $7f,$09,$19,$29,$46,$00;R
.db $46,$49,$49,$49,$31,$00;S
.db $01,$01,$7f,$01,$01,$00;T
.db $3f,$40,$40,$40,$3f,$00;U
.db $1f,$20,$40,$20,$1f,$00;V
.db $3f,$40,$38,$40,$3f,$00;W
.db $63,$14,$08,$14,$63,$00;X
.db $07,$08,$70,$08,$07,$00;Y
.db $61,$51,$49,$45,$43,$00;Z
.db $00,$7f,$41,$41,$00,$00;[
.db $55,$2a,$55,$2a,$55,$00;55
.db $00,$41,$41,$7f,$00,$00;]
.db $04,$02,$01,$02,$04,$00;^
.db $40,$40,$40,$40,$40,$00;_
.db $00,$01,$02,$04,$00,$00;`
.db $20,$54,$54,$54,$78,$00;a
.db $7f,$48,$44,$44,$38,$00;b
.db $38,$44,$44,$44,$20,$00;c
.db $38,$44,$44,$48,$7f,$00;d
.db $38,$54,$54,$54,$18,$00;e
.db $08,$7e,$09,$01,$02,$00;f
.db $0c,$52,$52,$52,$3e,$00;g
.db $7f,$08,$04,$04,$78,$00;h
.db $00,$44,$7d,$40,$00,$00;i
.db $20,$40,$44,$3d,$00,$00;j
.db $7f,$10,$28,$44,$00,$00;k
.db $00,$41,$7f,$40,$40,$00;l
.db $7c,$04,$18,$04,$78,$00;m
.db $7c,$08,$04,$04,$78,$00;n
.db $38,$44,$44,$44,$38,$00;o
.db $7c,$14,$14,$14,$08,$00;p
.db $08,$14,$14,$18,$7c,$00;q
.db $7c,$08,$04,$04,$08,$00;r
.db $48,$54,$54,$54,$20,$00;s
.db $04,$3f,$44,$40,$20,$00;t
.db $3c,$40,$40,$20,$7c,$00;u
.db $1c,$20,$40,$20,$1c,$00;v
.db $3c,$40,$30,$40,$3c,$00;w
.db $44,$28,$10,$28,$44,$00;x
.db $0c,$50,$50,$50,$3c,$00;y
.db $44,$64,$54,$4c,$44,$00;z
.db $7c,$20,$20,$1c,$20,$00;mju ($7B)
.db $5c,$62,$02,$62,$5c,$00;omega ($7C)

;    THAT'S ALL, GIRLS.... 
;but here may be even Arabish letters(in the next adresses (codes>$7C))
; or Japanese :-) :-)
